/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.DashboardClient");

dojo.require("com.ibm.team.dashboard.web.client.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardItem");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

/*
 * Aliases
 */
var Constants = com.ibm.team.dashboard.web.client.internal.Constants;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardItem = com.ibm.team.dashboard.web.client.internal.DashboardItem;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

/*
 * Public static fields and methods
 */
com.ibm.team.dashboard.web.client.internal.DashboardClient = {
	
	/*
	 * Creates a new dashboard scoped to the given item.
	 * 
	 * @param scope the scope of the dashboard (e.g. DashboardConstants.SCOPE_CONTRIBUTOR).
	 * @param scopeItemId the id of the item to associate the dashboard with
	 * 		(contributor, team area, or project area).
	 * @param srh the service response handler.
	 * @return a new Dashboard (asynchronously)
	 */
	createDashboard: function(/* number */ scope, /* string */ scopeItemId, /* boolean */ isPublic, /* ServiceResponseHandler */ srh) {
		// proxy srh; process result before returning
		var self = this;
		var serviceObjDash = {
			success: function(/* ResponseDTO */ responseDTO) {
				if (responseDTO.status == "OK" || responseDTO.status == "STALE") {
					responseDTO.data = new DashboardItem(responseDTO.data, self);					
				}
				srh.doCallback(responseDTO);
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		
		/*
		 * Get the process area that will govern this dashboard. For team dashboards
		 * it is the scope team area, otherwise it is the current project area.
		 */ 
		var processAreaItemId;
		if (scope == DashboardConstants.SCOPE_TEAM_AREA) {
			processAreaItemId = scopeItemId;
		}
		else {
			processAreaItemId = getCurrentProjectArea().itemId;
		}
		
		var srh2 = new ServiceResponseHandler(serviceObjDash, "success", "failure");
		var params = {processAreaItemId: processAreaItemId, scope: scope, scopeItemId: scopeItemId};
		if (isPublic)
			params.isPublic = isPublic;
		var req = new ServiceRequest(Constants.SERVICE_URI, "postCreateDashboard", params);
		TeamServerClient.invokeService(req, srh2);
	},
	
	/*
	 * Deletes a dashboard by item id, and returns the deleted dashboard's descriptor.
	 * 
	 * @param dashboardItemId the id of the dashboard item.
	 * @param srh the service response handler.
	 * @return the deleted dashboard's descriptor (asynchronously)
	 */
	deleteDashboard: function(/* number */ scope, /* string */ scopeItemId, /* boolean */ isPublic, /* string */ modified, /* boolean */ force, /* ServiceResponseHandler */ srh) {
		var params = {scope: scope, scopeItemId: scopeItemId};
		params.isPublic = isPublic ? true : false;
		params.force = force ? true : false;
		params.projectAreaItemId = getCurrentProjectArea().itemId;
		if (modified) {
			params.modified = modified;
		}
		var req = new ServiceRequest(Constants.SERVICE_URI, "postDeleteDashboard", params);
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Searches for dashboards.
	 * 
	 * @param query the search query text
	 * @param queryId the search query id, which will be returned in the results response
	 * @param includeTeams whether or not to include teams in the search
	 * @param includeContributors whether or not to include contributors in the search
	 * @return a SearchResultsDTO object containing the results (asynchronously)
	 */
	search: function(/* string */ query, /* string */ queryId, /* boolean */ includeTeams, /* boolean */ includeContributors, /* ServiceResponseHandler */ srh) {
		var req = new ServiceRequest(Constants.SERVICE_URI, "postSearch", {query: query, queryId: queryId, includeTeams: includeTeams, includeContributors: includeContributors, projectAreaItemId: getCurrentProjectArea().itemId});
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Generates a new UUID on the server side and returns it as a string
	 * to the handler.
	 */
	generateUUID: function(/* ServiceResponseHandler */ srh) {
		var req = new ServiceRequest(Constants.SERVICE_URI, "postGenerateUUID", null);
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Retrieves a dashboard by item id.
	 * 
	 * @param args an object with one of the following properties set: dashboardItemId, contributorItemId, teamItemId, projectItemId.
	 * @param srh the service response handler.
	 * @return a new Dashboard (asynchronously)
	 */
	getDashboard: function(/* object */ args, /* boolean */ includeDefinitions, /* ServiceResponseHandler */ srh) {
		// proxy srh; process result before returning
		var self = this;
		var serviceObj = {
			success: function(/* ResponseDTO */ responseDTO) {
				if (responseDTO.status == "OK" && responseDTO.data) {
					responseDTO.data = new DashboardItem(responseDTO.data, self);
				}
				srh.doCallback(responseDTO);
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		if (!args) args = new Object();
		args.includeDefinitions = includeDefinitions ? true : false;
		args.projectAreaItemId = getCurrentProjectArea().itemId;
		var srh2 = new ServiceResponseHandler(serviceObj, "success", "failure");
		var req = new ServiceRequest(Constants.SERVICE_URI, "getDashboard", args);
		TeamServerClient.invokeService(req, srh2);
	},
	
	/*
	 * Retrieves the last modified timestamp of the given dashboard.
	 * 
	 * @param dashboardItemId the item id of the dashboard whose timestamp to fetch
	 * @param srh the service response handler.
	 */
	getModified: function(/* string */ dashboardItemId, /* ServiceResponseHandler */ srh) {
		var req = new ServiceRequest(Constants.SERVICE_URI, "getModified", {dashboardItemId: dashboardItemId});
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Retrieves the information to create the left nav for the given user.
	 * 
	 * @param itemId the itemId of the user whose information is required.
	 * @param srh the service response handler.
	 */
	getLeftNavInfo: function(/* string */ itemId, /* ServiceResponseHandler */ srh) {
		var req = new ServiceRequest(Constants.SERVICE_URI, "getLeftNavInfo", {itemId: itemId, projectAreaItemId: getCurrentProjectArea().itemId});
		TeamServerClient.invokeService(req, srh);
	},
	
	/*
	 * Retrieves the information to create the viewlet settings area for the given user.
	 * 
	 * @param userId the id of the user whose information is required.
	 * @param srh the service response handler.
	 */
	getSettingsAreaInfo: function(/* string */ userId, /* number */ scope, /* string */ scopeItemId, /* string */ currentSelectedContributorItemId,/* ServiceResponseHandler */ srh) {
		var req = new ServiceRequest(Constants.SERVICE_URI, "getSettingsAreaInfo", {userId: userId, scope: scope, scopeItemId: scopeItemId, currentSelectedContributorItemId: currentSelectedContributorItemId});
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Retrieves the given viewlet definitions.
	 * 
	 * @param srh the service response handler.
	 */
	getViewletDefinitions: function(/* string[] */ definitionIds, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = new Object();
		if (definitionIds) {
			params.definitionIds = definitionIds;
		}
		var req = new ServiceRequest(Constants.SERVICE_URI, "getViewletDefinitions", params);
		if (tracker) {
			tracker.register(req, srh);
		}
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Retrieves the hierarchy of categorized viewlet types.
	 * 
	 * @param srh the service response handler.
	 */
	getViewletHierarchy: function(/* number */ scope, /* string */ scopeItemId, /* ServiceResponseHandler */ srh) {
		/*
		 * Get the process area that will govern this dashboard. For team dashboards
		 * it is the associated team area, otherwise it is the current project area.
		 */ 
		var processAreaItemId;
		if (scope == DashboardConstants.SCOPE_TEAM_AREA) {
			processAreaItemId = scopeItemId;
		}
		else {
			processAreaItemId = getCurrentProjectArea().itemId;
		}
		var req = new ServiceRequest(Constants.SERVICE_URI, "getViewletHierarchy", {processAreaItemId: processAreaItemId, scope: scope, scopeItemId: scopeItemId});
		TeamServerClient.invokeService(req, srh);
	},
	
	/*
	 * Gets the UUID for the given userId.
	 * 
	 * @param userId the id of the user whose UUID is required.
	 * @param srh the service response handler.
	 */
	getUserItemId: function(/* string */ userId, /* ServiceResponseHandler */ srh) {
		var req = new ServiceRequest(Constants.SERVICE_URI, "getUserItemId", {userId: userId});
		TeamServerClient.invokeService(req, srh);
	},
	
	/*
	 * Returns the information needed to display dashboard welcome.
	 * 
	 * @param userId the id of the user whose UUID is required.
	 * @param srh the service response handler.
	 */
	getWelcomeInfo: function(/* number */ scope, /* string */ userId, /* string */ teamName, /* ServiceResponseHandler */ srh) {
		var params = {scope: scope, projectAreaItemId: getCurrentProjectArea().itemId};
		if (userId) {
			params.userId = userId;
		}
		if (teamName) {
			params.teamName = teamName;
		}
		var req = new ServiceRequest(Constants.SERVICE_URI, "getWelcomeInfo", params);
		TeamServerClient.invokeService(req, srh);
	},
	
	getRelatedUsers: function(/* string */ userItemId, /* ServiceResponseHandler */ srh) {
		var params = { userItemId: userItemId, projectAreaItemId: getCurrentProjectArea().itemId };
		var req = new ServiceRequest(Constants.SERVICE_URI, "getRelatedUsers", params);
		TeamServerClient.invokeService(req, srh);
	},

	/*
	 * Locks a dashboard for a period of time, preventing others from
	 * modifying it or obtaining a lock. The lock remains in effect until
	 * the dashboard is unlocked, the lock is overriden by another user,
	 * or the keep-alive poll times out.
	 */
	lockDashboard: function(/* string */ dashboardItemId, /* boolean */ override, /* string */ modified, /* ServiceResponseHandler */ srh) {
		var req = new ServiceRequest(Constants.SERVICE_URI, "postLockDashboard", {dashboardItemId: dashboardItemId, override: override, modified: modified});
		TeamServerClient.invokeService(req, srh);
		
		// start keep-alive
		if (this._keepAlive) {
			clearInterval(this._keepAlive);
		}
		var self = this;
		this._keepAlive = setInterval(dojo.hitch(this, function() {
			var serviceObj = {
				success: function(/* ResponseDTO */ responseDTO) {
					if (responseDTO.status != "OK") {
						clearInterval(self._keepAlive);
					}
				},
				failure: function(error) {
					clearInterval(self._keepAlive);
				}
			};
			var srh2 = new ServiceResponseHandler(serviceObj, "success", "failure");
			var req2 = new ServiceRequest(Constants.SERVICE_URI, "postLockDashboard", {dashboardItemId: dashboardItemId, override: override, modified: modified});
			TeamServerClient.invokeService(req2, srh2);
		}), 30*1000);
	},
	
	/*
	 * Posts the given modifications on the given dashboard. Each modification
	 * object represents one modification to be performed on the dashboard, and
	 * has the form { type: <integer>, ... }. Type must be one of the
	 * Constants.MODIFICATION_TYPE_* constants, and the other properties are
	 * specific to the modification type - see DashboardModifier#applyModifications
	 * for expected properties.
	 */
	postModifications: function(/* string */ dashboardItemId, /* object[] */ modifications, /* string */ modified, /* ServiceResponseHandler */ srh) {
		var jsonArray = new Array();
		for (var i=0;i<modifications.length;i++) {
			jsonArray.push(dojo.toJson(modifications[i]));
		}
		var req = new ServiceRequest(Constants.SERVICE_URI, "postModifications", {dashboardItemId: dashboardItemId, modifications: jsonArray, modified: modified});
		TeamServerClient.invokeService(req, srh);
	},
	
	/*
	 * Unlocks a dashboard that was previously locked.
	 */
	unlockDashboard: function(/* string */ dashboardItemId) {
		var serviceObj = { empty: function() {} };
		var srh = new ServiceResponseHandler(serviceObj, "empty", "empty");
		var req = new ServiceRequest(Constants.SERVICE_URI, "postUnlockDashboard", {dashboardItemId: dashboardItemId});
		TeamServerClient.invokeService(req, srh);
		
		if (this._keepAlive) {
			clearInterval(this._keepAlive);
			delete this._keepAlive;
		}
	}
};

})();
